@@ -1,23 +1,25 @@
 /*
  * pgntags.c -- Functions to manage PGN tags
- * XBoard $Id: pgntags.c,v 1.11 1999/08/13 07:16:13 mann Exp $
  *
- * Copyright 1995 Free Software Foundation, Inc.
+ * Copyright 1995,2009 Free Software Foundation, Inc.
+ *
+ * Enhancements Copyright 2005 Alessandro Scotti
  *
  * ------------------------------------------------------------------------
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU XBoard is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or (at
+ * your option) any later version.
+ *
+ * GNU XBoard is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
+ * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA.
+ * along with this program. If not, see http://www.gnu.org/licenses/.  *
+ *
  * ------------------------------------------------------------------------
  *
  * This file could well be a part of backend.c, but I prefer it this
@@ -105,6 +107,10 @@ int ParsePGNTag(tag, gameInfo)
 	else
 	    gameInfo->result = GameUnfinished;
 	success = TRUE;
+    } else if (StrCaseCmp(name, "TimeControl") == 0) {
+//	int tc, mps, inc = -1;
+//	if(sscanf(value, "%d/%d", &mps, &tc) == 2 || )
+	success = StrSavePtr(value, &gameInfo->timeControl) != NULL;
     } else if (StrCaseCmp(name, "FEN") == 0) {
 	success = StrSavePtr(value, &gameInfo->fen) != NULL;
     } else if (StrCaseCmp(name, "SetUp") == 0) {
@@ -114,6 +120,9 @@ int ParsePGNTag(tag, gameInfo)
         /* xboard-defined extension */
         gameInfo->variant = StringToVariant(value);
 	success = TRUE;
+    } else if (StrCaseCmp(name, PGN_OUT_OF_BOOK) == 0) {
+        /* [AS] Out of book annotation */
+        success = StrSavePtr(value, &gameInfo->outOfBook) != NULL;
     } else {
 	if (gameInfo->extraTags == NULL) {
 	    oldTags = "";
@@ -249,5 +258,14 @@ ReplaceTags(tags, gameInfo)
 	    if (err != 0) return err;
 	} 
     }
+    /* just one problem...if there is a result in the new tags,
+     * DisplayMove() won't ever show it because ClearGameInfo() set
+     * gameInfo->resultDetails to NULL. So we must plug something in if there
+     * is a result.
+     */
+    if (gameInfo->result != GameUnfinished) {
+      if (gameInfo->resultDetails) free(gameInfo->resultDetails);
+      gameInfo->resultDetails = strdup("");
+    }
     return 0;
 }
