@@ -1,25 +1,27 @@
 /*
  * wclipbrd.c -- Clipboard routines for WinBoard
- * $Id: wclipbrd.c,v 2.1 2003/10/27 19:21:02 mann Exp $
  *
- * Copyright 2000 Free Software Foundation, Inc.
+ * Copyright 2000,2009 Free Software Foundation, Inc.
+ *
+ * Enhancements Copyright 2005 Alessandro Scotti
  *
  * ------------------------------------------------------------------------
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU XBoard is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or (at
+ * your option) any later version.
+ *
+ * GNU XBoard is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
+ * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- * ------------------------------------------------------------------------
- */
+ * along with this program. If not, see http://www.gnu.org/licenses/.  *
+ *
+ *------------------------------------------------------------------------
+ ** See the file ChangeLog for a revision history.  */
 
 #include "config.h"
 
@@ -30,13 +32,14 @@
 #include <sys/stat.h>
 
 #include "common.h"
-#include "winboard.h"
 #include "frontend.h"
 #include "backend.h"
+#include "winboard.h"
 #include "wclipbrd.h"
 
 /* Imports from winboard.c */
 extern HWND hwndMain;
+Boolean ParseFEN(Board b, int *stm, char *FEN);
 
 /* File globals */
 static char *copyTemp;
@@ -47,7 +50,8 @@ CopyFENToClipboard()
 {
   char *fen = NULL;
 
-  fen = PositionToFEN(currentMove);
+  if(gameMode == EditPosition) EditPositionDone(TRUE); // [HGM] mak sure castling rights are set consistently
+  fen = PositionToFEN(currentMove, NULL);
   if (!fen) {
     DisplayError("Unable to convert position to FEN.", 0);
     return;
@@ -57,6 +61,38 @@ CopyFENToClipboard()
   free(fen);
 }
 
+/* [AS] */
+HGLOBAL ExportGameListAsText();
+
+VOID CopyGameListToClipboard()
+{
+    HGLOBAL hMem = ExportGameListAsText();
+    
+    if( hMem != NULL ) {
+        /* Assign memory block to clipboard */
+        BOOL ok = OpenClipboard( hwndMain );
+
+        if( ok ) {
+            ok = EmptyClipboard();
+
+            if( ok ) {
+                if( hMem != SetClipboardData( CF_TEXT, hMem ) ) {
+                    ok = FALSE;
+                }
+            }
+
+            CloseClipboard();
+
+            if( ! ok ) {
+                GlobalFree( hMem );
+            }
+        }
+
+        if( ! ok ) {
+            DisplayError( "Cannot copy list to clipboard.", 0 );
+        }
+    }
+}
 
 VOID
 CopyGameToClipboard()
@@ -165,7 +201,7 @@ CopyTextToClipboard(char *text)
     locked = !((err == NO_ERROR) || (err == ERROR_NOT_LOCKED));
     if (appData.debugMode) {
       fprintf(debugFP, 
-	      "CopyTextToClipboard(): err %d locked %d\n", err, locked);
+	      "CopyTextToClipboard(): err %d locked %d\n", (int)err, locked);
     }
   }
   if (locked) {
@@ -194,6 +230,16 @@ CopyTextToClipboard(char *text)
   return TRUE;
 }
 
+/* [AS] Reworked paste functions so they can work with strings too */
+
+VOID PasteFENFromString( char * fen )
+{
+  if (appData.debugMode) {
+    fprintf(debugFP, "PasteFenFromString(): fen '%s'\n", fen);
+  }
+  EditPositionPasteFEN(fen); /* call into backend */
+  free(fen);
+}
 
 
 VOID
@@ -204,11 +250,31 @@ PasteFENFromClipboard()
       DisplayError("Unable to paste FEN from clipboard.", 0);
       return;
   }
-  if (appData.debugMode) {
-    fprintf(debugFP, "PasteFenFromClipboard(): fen '%s'\n", fen);
+  PasteFENFromString( fen );
+}
+
+VOID PasteGameFromString( char * buf )
+{
+  FILE *f;
+  size_t len;
+  if (!pasteTemp) {
+    pasteTemp = tempnam(NULL, "wbpt");
   }
-  EditPositionPasteFEN(fen); /* call into backend */
-  free(fen);
+  f = fopen(pasteTemp, "w");
+  if (!f) {
+    DisplayError("Unable to create temporary file.", 0);
+    free(buf); /* [AS] */
+    return;
+  }
+  len = fwrite(buf, sizeof(char), strlen(buf), f);
+  fclose(f);
+  if (len != strlen(buf)) {
+    DisplayError("Error writing to temporary file.", 0);
+    free(buf); /* [AS] */
+    return;
+  }
+  LoadGameFromFile(pasteTemp, 0, "Clipboard", TRUE);
+  free( buf ); /* [AS] */
 }
 
 
@@ -218,28 +284,31 @@ PasteGameFromClipboard()
   /* Write the clipboard to a temp file, then let LoadGameFromFile()
    * do all the work.  */
   char *buf;
-  FILE *f;
-  size_t len;
   if (!PasteTextFromClipboard(&buf)) {
     return;
   }
-  if (!pasteTemp) {
-    pasteTemp = tempnam(NULL, "wbpt");
-  }
-  f = fopen(pasteTemp, "w");
-  if (!f) {
-    DisplayError("Unable to create temporary file.", 0);
-    return;
-  }
-  len = fwrite(buf, sizeof(char), strlen(buf), f);
-  fclose(f);
-  if (len != strlen(buf)) {
-    DisplayError("Error writing to temporary file.", 0);
-    return;
-  }
-  LoadGameFromFile(pasteTemp, 0, "Clipboard", TRUE);
+  PasteGameFromString( buf );
 }
 
+/* [AS] Try to detect whether the clipboard contains FEN or PGN data */
+VOID PasteGameOrFENFromClipboard()
+{
+  char *buf;
+//  char *tmp;
+  Board dummyBoard; int dummy; // [HGM] paste any
+
+  if (!PasteTextFromClipboard(&buf)) {
+    return;
+  }
+
+  // [HGM] paste any: make still smarter, to allow pasting of games without tags, recognize FEN in stead
+  if(!ParseFEN(dummyBoard, &dummy, buf) ) {
+      PasteGameFromString( buf );
+  }
+  else {
+      PasteFENFromString( buf );
+  }
+}
 
 int 
 PasteTextFromClipboard(char **text)
@@ -281,15 +350,11 @@ PasteTextFromClipboard(char **text)
     fprintf(debugFP, "PasteTextFromClipboard(): lock count %d\n", lockCount);
   }
   SetLastError(NO_ERROR);
-#if 1
   /*suggested by Wilkin Ng*/
   lockCount = GlobalFlags(hClipMem) & GMEM_LOCKCOUNT;
   if (lockCount) {
     locked = GlobalUnlock(hClipMem);
   }
-#else
-  locked = GlobalUnlock(hClipMem);
-#endif
   err = GetLastError();
   if (appData.debugMode) {
     lockCount = GlobalFlags(hClipMem) & GMEM_LOCKCOUNT;
@@ -299,7 +364,7 @@ PasteTextFromClipboard(char **text)
     locked = !((err == NO_ERROR) || (err == ERROR_NOT_LOCKED));
     if (appData.debugMode) {
       fprintf(debugFP, 
-	      "PasteTextFromClipboard(): err %d locked %d\n", err, locked);
+	      "PasteTextFromClipboard(): err %d locked %d\n", (int)err, locked);
     }
   }
   if (locked) 
