@@ -230,6 +230,10 @@ static void InsertIntoMemo( int which, char * text )
 {
 	Arg arg; XawTextBlock t; Widget edit;
 
+	/* the backend adds \r\n, which is needed for winboard, 
+	 * for xboard we delete them again over here */
+	if(t.ptr = strchr(text, '\r')) *t.ptr = ' ';
+
 	t.ptr = text; t.firstPos = 0; t.length = strlen(text); t.format = XawFmt8Bit;
 	edit = XtNameToWidget(engineOutputShell, which ? "*form2.text" : "*form.text");
 	XawTextReplace(edit, 0, 0, &t);
@@ -851,7 +855,8 @@ static void UpdateControls( EngineOutputData * ed )
             strncpy( mov, ed->hint, sizeof(mov) );
             mov[ sizeof(mov)-1 ] = '\0';
 
-            sprintf( buf, "%d/%d: %s [%02d:%02d:%02d]", ed->an_move_index, ed->an_move_count, mov, time_mins / 60, time_mins % 60, time_secs % 60 );
+	    sprintf( buf, "[%d] %d/%d: %s [%02d:%02d:%02d]", ed->depth, ed->an_move_index,
+			ed->an_move_count, mov, time_mins / 60, time_mins % 60, time_secs % 60 );
         }
 
         SetEngineState( ed->which, STATE_ANALYZING, buf );
